/***************************************************************************
  $Id: TLAgentApplet.java,v 1.6 1998/10/13 18:18:32 toma Exp $
***************************************************************************/


package webwisdom.tango;


import java.applet.Applet;
import java.util.Vector;

import webwisdom.tango.consts.CAConsts;
import webwisdom.tango.consts.Const;
import webwisdom.tango.messages.AppEventMessage;
import webwisdom.tango.structures.AppletHandle;
import webwisdom.tango.structures.LocalBase;
import webwisdom.tango.interfaces.AppletBaseInterf;

import netscape.javascript.JSObject;


/**Tango system proxy for applets.
   This class may be used to create TAgent.
   *@see TAgent*/
public final class TLAgentApplet implements TLAgent,AppletBaseInterf
{
    private static final String CL="TLAgentApplet";


    private Applet applet=null;
    private AppletHandle handle=null;
    private TLListener lowlList;


    /**Registers an applet in the Tango system.
     *@param app applet to be registered
     *@exception TangoException if cannot get reference to Tango*/
    public TLAgentApplet(Applet app) throws TangoException
    {
      if(app==null)
	throw new IllegalArgumentException("null applet");
      applet=app;
      
      try
      {
	handle=register(this);
      }
      catch(UnsatisfiedLinkError e)
      {
	throw new TangoException("Not in Netscape");	
      }
      catch(NoClassDefFoundError e)
      {
	throw new TangoException("Not in Netscape");	
      }
      if(handle==null)
	throw new TangoException("Tango demon not found");
    }// TLAgentApplet _____________________________________________________


    public void exit()
    {
      //System.out.println(CL+".exit()");
      handle.remove();
      handle=null;
    }// exit ______________________________________________________________


    public void add(TLListener tl)
    {
      lowlList=tl;
    }// add _______________________________________________________________

  
    public final void send(AppEventMessage m)
    {
      handle.sendAppToOther(m);
    }// send ______________________________________________________________


    public void send(int userlist[],AppEventMessage m)
    {
      handle.sendAppToOther(userlist,m);
    }// send ______________________________________________________________


    public void receive(AppEventMessage m)
    {
      if(lowlList!=null)
	lowlList.receive(m);
    }// receive ___________________________________________________________


    public JSObject getJSWindow()
    {
      try
      {
	return (JSObject)((JSObject)JSObject.getWindow(applet)).getMember("top");
      }
      catch(Exception e)
      {
	return null;
      }
    }// getJSWindow _______________________________________________________


    static AppletHandle register(AppletBaseInterf base)
    {
      netscape.security.PrivilegeManager.enablePrivilege("UniversalBrowserAccess");
      int AID,SID,AT;
      {
	JSObject win=(JSObject)base.getJSWindow();
	String name=(String)win.getMember("name");	
	//System.out.println(CL+".register(Applet): win="+win+",name=\""+name+"\"");
	if((name==null)||(name.length()==0))
	{
	  System.err.println(CL+".register(): corrupted window name!");
	  return null;
	}
	//System.out.println(CL+".register(): name=\""+name+"\"");
	
	AID=0;
	try
	{
	  int AIDend=name.indexOf('_');
	  int SIDend=name.indexOf('_',AIDend+1);
	  AID=Integer.parseInt(name.substring(3,AIDend));
	  SID=Integer.parseInt(name.substring(AIDend+1,SIDend));
	  AT=Integer.parseInt(name.substring(SIDend+1));
	  
	  System.out.println(CL+".register(): AID="+AID+",SID="+SID+",AT="+AT);
	}
	catch(Exception e)
	{
	  System.err.println(CL+".register(): corrupted window name!: "+e);
	  //e.printStackTrace();
	  return null;
	}
      }
      try
      {
	return LocalBase.getLocalBase().reg.registerApplet(base,AID,SID,AT);
      }
      catch(Exception e)
      {
	System.err.println(CL+".register(): plugin not found!");     
	return null;
      }
    }// register __________________________________________________________


    public String toString()
    {
      return CL+"[applet="+applet+",handle="+handle+"]";
    }// toString ___________________________________________________________


}// TLAgentApplet /////////////////////////////////////////////////////////

  
